
----------------------------------------------------------------
-- write by xerysherry

-- 图片和画布管理

local lf = love.filesystem;
local li = love.image;
local lg = love.graphics;

-- 得到基类
local Object = getClass("Object");
local Size = getClass("Object.Size");
local Font = getClass("Object.FontManager");

-- 图片对象
local Image = class("Image", Object,
{
	path = nil,			--路径
	img = nil,			--图片
});

-- 画布对象
local Canvas = class("Canvas", Object,
{
	size = nil,
	canvas = nil,
	-- 成员方法
	initialize = function(this, w, h)
		this.size = Size:new(w, h);
		this.canvas = lg.newCanvas(w, h);
	end,
	getSize = function (this)
		return this.size:getSize();
	end,
})

-- 绘制管理器
local Graphics = class("Graphics", Object, 
{
	--列表
	images = nil,
	canvas = nil,
	sprites = nil,
	particleSystems = nil,
	quads = nil,
	fonts = Font:new("font"),
	--枚举
	drawtype = {
		image = 0,
		canvas = 1,
		sprite = 2,
		particleSystem = 3,
	},
});

function Graphics:initialize(...)
	self.images = {};
	self.canvas = {};
	self.sprites = {};
	self.particleSystems = {};
	self.quads = {};
end

----------------------------------------------------------------
-- 创建区域
function Graphics:createQuad(idx, x, y, width, height, sw, sh)
	if self.quads[idx]==nil then
		self.quads[idx] = lg.newQuad(x, y, width, height, sw, sh);
	end
end

function Graphics:getQuad(idx)
	return self.quads[idx];
end

function Graphics:deleteQuad(idx)
	self.quads[idx] = nil;
end

----------------------------------------------------------------
-- 获取相同路径的图片（可以用来确认图片是否已经加载成功）
function Graphics:getSamePathImage(path)
	if path==nil then				--path为nil是没有意义的
		return nil;
	end
	
	for i,v in pairs(self.images) do
		if path==v.path then
			return v;
		end
	end
	return nil;
end

-- 读取一张图片
function Graphics:createImageFromPath(idx, path)
	self.images[idx] = self:getSamePathImage(path);	--确保同一个文件内存中只有一份
	if self.images[idx]==nil then
		self.images[idx] = Image:new();
		self.images[idx].path = path;
		self.images[idx].img = lg.newImage(path);
	end
end

-- 通过数据流(binary or base64)创建一张图片
function Graphics:createImageFromStream(idx, data, decoder)
	if decoder==nil then
		decoder = "file";
	end
	local fd = lf.newFileData(data, idx, decoder);
	local id = li.newImageData(fd);
	
	self.images[idx] = Image:new();
	self.images[idx].img = lg.newImage(id);
end

-- 读取一张图片（使用lua的filesystem读取，
-- 支持相对路径和全路径，在需要包外资源时使用之）
function Graphics:createImageFromPathPlus(idx, path)
	local imgfile = assert(io.open(path, "rb")); 
	local data = imgfile:read("*all");
	imgfile:close();
	
	self:createImageFromStream(idx,	 data, "file");
	data = nil;
end

-- 通过画布创建一张图片
function Graphics:createImageFromCanvas(idx, canvas)
	self.images[idx] = Image:new();
	self.images[idx].img = lg.newImage(canvas:getImageData());
end

-- 通过画布索引（同处于一个Graphics实例中）创建一张图片
function Graphics:createImageFromCanvas(idx, canvas_idx)
	self.images[idx] = Image:new();
	self.images[idx].img = lg.newImage(self.canvas[canvas_idx]:getImageData());
end

-- 获得图片尺寸
function Graphics:getImageSize(idx)
	if self.images[idx]==nil then
		return nil, nil;
	end
	return self.images[idx].img:getWidth(), self.images[idx].img:getHeight();
end

-- 返回图片
function Graphics:getImage(idx)
	return self.images[idx].img;
end

-- 删除图片
function Graphics:deleteImage(idx)
	self.images[idx] = nil;
end

----------------------------------------------------------------
-- 创建画布
function Graphics:createCanvas(idx, width, height)
	self.canvas[idx] = Canvas:new(width, height);
end

-- 返回画布
function Graphics:getCanvas(idx)
	return self.canvas[idx].canvas;
end

-- 删除画布
function Graphics:deleteCanvas(idx)
	self.canvas[idx] = nil;
end

-- 获得画布尺寸
function Graphics:getCanvasSize(idx)
	if self.canvas[idx]==nil then
		return nil, nil;
	end
	return self.canvas[idx]:getSize();
end

----------------------------------------------------------------
-- 创建精灵(图片必须已经创建，并且在同一个对象内)
function Graphics:createSprite(idx, image_idx, count)
	assert(self.images[image_idx]~=nil, image_idx.." is not image");
	self.sprites[idx] = lg.newSpriteBatch(self.images[image_idx], count);
end

-- 添加精灵
function Graphics:spriteAdd(idx, x, y, r, sx, sy, ox, oy)
	assert(self.sprites[idx]~=nil, idx.." is not sprite");
	self.sprites[idx]:add(x, y, r, sx, sy, ox, oy);
end

function Graphics:spriteAddq(idx, quad_idx, x, y, r, sx, sy, ox, oy)
	assert(self.sprites[idx]~=nil, idx.." is not sprite");
	assert(self.quads[quad_idx]~=nil, quad_idx.." is not quad");
	self.sprites[idx]:addq(self.quads[quad_idx], x, y, r, sx, sy, ox, oy);
end

-- 清理精灵
function Graphics:spriteClear(idx)
	assert(self.sprites[idx]~=nil, idx.." is not sprite");
	self.sprites[idx]:clear();
end

-- 改变图片
function Graphics:spriteChangeImage(idx, image_idx)
	assert(self.images[image_idx]~=nil, image_idx.." is not image");
	assert(self.sprites[idx]~=nil, idx.." is not sprite");
	self.sprites[idx]:setImage(self.images[image_idx]);
end

-- 返回精灵
function Graphics:getSprite(idx)
	return self.sprites[idx];
end

-- 删除精灵
function Graphics:deleteSprite(idx)
	self.sprites[idx] = nil;
end

----------------------------------------------------------------
-- 创建粒子系统
function Graphics:createParticleSystem(idx, image_idx, count)
	assert(self.images[image_idx]~=nil, image_idx.." is not image");
	self.particleSystems[idx] = lg.newParticleSystem(self.images[image_idx].img, count);
end

-- 返回粒子系统
function Graphics:getParticleSystem(idx)
	return self.particleSystems[idx];	
end

-- 删除粒子系统
function Graphics:deleteParticleSystem(idx)
	self.particleSystems[idx] = nil;
end

----------------------------------------------------------------
-- 创建字体
function Graphics:createFontFromPath(idx, path, size)
	self.fonts:createFontFromPath(idx, path, size);
end

function Graphics:createFontFromSize(idx, size)
	self.fonts:createFontFromSize(idx, size);
end

function Graphics:createFontFromPathPlus(idx, path, size)
	self.fonts:createFontFromPathPlus(idx, path, size);
end

-- 返回字体
function Graphics:getFont(idx)
	return self.fonts:getFont(idx);
end

-- 设置字体
function Graphics:setFont(idx)
	self.fonts:setFont(idx);
end

----------------------------------------------------------------
-- encode
-- 0.72 is huger different than 0.80
-- function Graphics:encode(idx, t)

-- end

----------------------------------------------------------------
-- 直接获取love.graphics的方法
-- (同名后缀加"_")

-- 绘制方法
Graphics._clear = lg.clear;
Graphics._circle = lg.circle;
Graphics._line = lg.line;
Graphics._point = lg.point;
Graphics._polygon = lg.polygon;
Graphics._print = lg.print;
Graphics._printf = lg.printf;
Graphics._quad = lg.quad;
Graphics._rectangle = lg.rectangle;
Graphics._triangle = lg.triangle;
Graphics._draw = lg.draw;
Graphics._drawq = lg.drawq;
-- 绘制目标
Graphics._getRenderTarget = lg.getRenderTarget;
Graphics._setRenderTarget = lg.setRenderTarget;
-- 混合模式
Graphics._getBlendMode = lg.getBlendMode;
Graphics._setBlendMode = lg.setBlendMode;
-- 颜色和颜色模式
Graphics._getColorMode = lg.getColorMode;
Graphics._setColorMode = lg.setColorMode;
Graphics._getColor = lg.getColor;
Graphics._setColor = lg.setColor;
-- 视口大小
Graphics._getWidth = lg.getWidth;
Graphics._getHeight = lg.getHeight;
-- 背景颜色相关
Graphics._getBackgroundColor = lg.getBackgroundColor;
Graphics._setBackgroundColor = lg.setBackgroundColor;
-- 裁剪区相关
Graphics._getScissor = lg.getScissor;
Graphics._setScissor = lg.setScissor;
-- 保存和读取当前转换
Graphics._push = lg.push;
Graphics._pop = lg.pop;
-- 转换
Graphics._rotate = lg.rotate;
Graphics._scale = lg.scale;
Graphics._translate = lg.translate;

----------------------------------------------------------------

function Graphics.rectangle(mode, rect)
	lg.rectangle(mode, rect.x, rect.y, rect.width, rect.height);
end

-- 切合渲染目标
function Graphics:setCanvas(canvas_idx)
	if canvas_idx==nil then
		--lg.setRenderTarget();
		lg.setCanvas();
	else
		assert(self.canvas[canvas_idx]~=nil, canvas_idx.." is not canvas");
		--lg.setRenderTarget(self.canvas[canvas_idx]);
		lg.setCanvas(self:getCanvas(canvas_idx));
	end
end

-- 绘制
function Graphics:drawImage(idx, x, y, r, sx, sy, ox, oy)
	local img = self:getImage(idx);
	assert(img~=nil, idx.." is not image");
	lg.draw(img, x, y, r, sx, sy, ox, oy);
end

function Graphics:drawCanvas(idx, x, y, r, sx, sy, ox, oy)
	local canvas = self:getCanvas(idx);
	assert(canvas~=nil, idx.." is not canvas");
	lg.draw(canvas, x, y, r, sx, sy, ox, oy);
end

function Graphics:drawSprite(idx, x, y, r, sx, sy, ox, oy)
	local sprite = self:getSprite(idx);
	assert(sprite~=nil, idx.." is not sprite");
	lg.draw(sprite, x, y, r, sx, sy, ox, oy);
end

function Graphics:drawParticleSystem(idx, x, y, r, sx, sy, ox, oy)
	local ps = self:getParticleSystem(idx);
	assert(ps~=nil, idx.." is not particleSystems");
	lg.draw(ps, x, y, r, sx, sy, ox, oy);
end

-- 绘制 t为Graphics.drawtype
function Graphics:draw(t, idx, x, y, r, sx, sy, ox, oy)
	if t==self.drawtype.image then
		self:drawImage(idx, x, y, r, sx, sy, ox, oy);
	elseif t==self.drawtype.canvas then
		self:drawCanvas(idx, x, y, r, sx, sy, ox, oy);
	elseif t==self.drawtype.sprite then
		self:drawSprite(idx, x, y, r, sx, sy, ox, oy);
	elseif t==self.drawtype.particleSystem then
		self:drawParticleSystem(idx, x, y, r, sx, sy, ox, oy);
	end
end

function Graphics:drawqImage(idx, quad_idx, x, y, r, sx, sy, ox, oy)
	local img = self:getImage(idx);
	assert(img~=nil, idx.." is not image");
	local quad = self:getQuad(quad_idx);
	assert(quad~=nil, quad_idx.." is not quad");
	lg.drawq(img, quad, 
		x, y, r, sx, sy, ox, oy);
end

function Graphics:drawqCanvas(idx, quad_idx, x, y, r, sx, sy, ox, oy)
	local canvas = self:getCanvas(idx);
	assert(canvas~=nil, idx.." is not image");
	local quad = self:getQuad(quad_idx);
	assert(quad~=nil, quad_idx.." is not quad");
	lg.drawq(canvas, quad, 
		x, y, r, sx, sy, ox, oy);
end

function Graphics:drawqSprite(idx, quad_idx, x, y, r, sx, sy, ox, oy)
	local sprite = self:getSprite(idx);
	assert(sprite~=nil, idx.." is not image");
	local quad = self:getQuad(quad_idx);
	assert(quad~=nil, quad_idx.." is not quad");
	lg.drawq(sprite, quad, 
		x, y, r, sx, sy, ox, oy);
end

function Graphics:drawqParticleSystem(idx, quad_idx, x, y, r, sx, sy, ox, oy)
	local ps = self:getParticleSystem(idx);
	assert(ps~=nil, idx.." is not image");
	local quad = self:getQuad(quad_idx);
	assert(quad~=nil, quad_idx.." is not quad");
	lg.drawq(ps, quad, 
		x, y, r, sx, sy, ox, oy);
end

-- 绘制 t为Graphics.drawtype
function Graphics:drawq(t, idx, quad_idx, x, y, r, sx, sy, ox, oy)
	if t==self.drawtype.image then
		self:drawqImage(idx, quad_idx, x, y, r, sx, sy, ox, oy);
	elseif t==self.drawtype.canvas then
		self:drawqCanvas(idx, quad_idx, x, y, r, sx, sy, ox, oy);
	elseif t==self.drawtype.sprite then
		self:drawqSprite(idx, quad_idx, x, y, r, sx, sy, ox, oy);
	elseif t==self.drawtype.particleSystem then
		self:drawqParticleSystem(idx, quad_idx, x, y, r, sx, sy, ox, oy);
	end
end

return Graphics;
